#include "PlayingState.h"
#include "GameData.h"
#include "GameStateManager.h"
#include "GameOver.h"
#include "NewLevel.h"

PlayingState::PlayingState( GameData &Data )
	:
	GameState( Data ),
	playerUpdate( delay )
{
}

void PlayingState::Update( float DeltaTime )
{
	const int tileSize = m_data.brd.GetTileSize();

	{
		bool transitioned = false;
		m_data.player.UserInput( m_data.m_keyboard );
		playerUpdate.Update( DeltaTime, [ this, &transitioned ]
		{
			const int oldX = m_data.player.GetX();
			const int oldY = m_data.player.GetY();

			m_data.player.Update();

			const int y = m_data.player.GetY();
			const int x = m_data.player.GetX();

			const auto curTileType = m_data.brd.GetTileType( x, y );

			switch( curTileType )
			{
				case Board::Type::tail:
					//player dies if he collides with his tail
					failSound.Play();
					m_data.m_stateManager.Transition( std::make_unique<GameOver>( m_data ) );
					transitioned = true;
					return;
				case Board::Type::filled:
					//player stops moving when he goes from inside to outside
					if( m_data.brd.GetTileType( oldX, oldY ) == Board::Type::tail )
					{
						m_data.player.Stop();
					}
					break;
				case Board::Type::empty:
					//creates the tail
					m_data.brd.SetTileType( x, y, Board::Type::tail );
					break;
			}
		} );

		if( transitioned ) return;
	}

	//move inside enemies
	for( int i = 0; i < m_data.enemyCount; i++ )
	{
		m_data.enemy[ i ].Move( m_data.brd, tileSize, DeltaTime );
	}

	//move outside enemy
	m_data.outEnemy.Move( m_data.brd, DeltaTime );

	//if player collides with outside enemy
	if( Collides( float( tileSize ) ) )
	{
		failSound.Play();
		m_data.m_stateManager.Transition( std::make_unique<GameOver>( m_data ) );
		return;
	}

	if( m_data.brd.grid[ m_data.player.GetY() ][ m_data.player.GetX() ] == Board::Type::filled )
	{
		for( int i = 0; i < m_data.enemyCount; i++ )
		{
			m_data.brd.Drop( 
				int( m_data.enemy[ i ].GetY() ) / tileSize, 
				int( m_data.enemy[ i ].GetX() ) / tileSize
			);
		}

		m_data.brd.Collapse();
	}

	//player dies if enemy collides with tail
	for( int i = 0; i < m_data.enemyCount; i++ )
	{
		if( m_data.brd.GameOver( 
			int( m_data.enemy[ i ].GetY() ), 
			int( m_data.enemy[ i ].GetX() ) 
		) )
		{
			failSound.Play();
			m_data.m_stateManager.Transition( std::make_unique<GameOver>( m_data ) );
			return;
		}
	}

	if( m_data.brd.LevelComplete() )
	{		
		clearSound.Play();
		m_data.m_stateManager.Transition( std::make_unique<NewLevel>( m_data ) );
		return;
	}
}

void PlayingState::ComposeFrame( Graphics & Gfx )
{
	m_data.player.Draw( Gfx, m_data.brd.GetTileSize(), m_data.brd.grid );
	m_data.outEnemy.Draw( Gfx );
}

bool PlayingState::Collides( float ts )const
{
	const float x = float( m_data.player.GetX() );
	const float y = float( m_data.player.GetY() );
	const float left = m_data.outEnemy.GetX() / ts;
	const float right = ( m_data.outEnemy.GetX() + ts ) / ts;
	const float top = m_data.outEnemy.GetY() / ts;
	const float bottom = ( m_data.outEnemy.GetY() + ts ) / ts;

	return( left <= ( x + 1.0f ) && right >= x &&
			top <= ( y + 1.0f ) && bottom >= y );
}
